import { IInstantiationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/instantiation";
import { ITextModel } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/model";
import { IEditorWorkerService } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/services/editorWorker.service";
import { Event } from "@codingame/monaco-vscode-api/vscode/vs/base/common/event";
import { URI } from "@codingame/monaco-vscode-api/vscode/vs/base/common/uri";
import { IFileService } from "@codingame/monaco-vscode-api/vscode/vs/platform/files/common/files.service";
import { IModelService } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/services/model.service";
import { ILanguageSelection } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/languages/language";
import { Disposable, IDisposable } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { ILogService } from "@codingame/monaco-vscode-api/vscode/vs/platform/log/common/log.service";
import { ILogEntry, IOutputContentSource, OutputChannelUpdateMode } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/output/common/output";
export declare function parseLogEntryAt(model: ITextModel, lineNumber: number): ILogEntry | null;
export interface IOutputChannelModel extends IDisposable {
    readonly onDispose: Event<void>;
    readonly source: IOutputContentSource | ReadonlyArray<IOutputContentSource>;
    getLogEntries(): ReadonlyArray<ILogEntry>;
    append(output: string): void;
    update(mode: OutputChannelUpdateMode, till: number | undefined, immediate: boolean): void;
    updateChannelSources(sources: ReadonlyArray<IOutputContentSource>): void;
    loadModel(): Promise<ITextModel>;
    clear(): void;
    replace(value: string): void;
}
interface IContentProvider {
    readonly onDidAppend: Event<void>;
    readonly onDidReset: Event<void>;
    reset(): void;
    watch(): void;
    unwatch(): void;
    getContent(): Promise<{
        readonly content: string;
        readonly consume: () => void;
    }>;
    getLogEntries(): ReadonlyArray<ILogEntry>;
}
export declare abstract class AbstractFileOutputChannelModel extends Disposable implements IOutputChannelModel {
    private readonly modelUri;
    private readonly language;
    private readonly outputContentProvider;
    protected readonly modelService: IModelService;
    private readonly editorWorkerService;
    private readonly _onDispose;
    readonly onDispose: Event<void>;
    protected loadModelPromise: Promise<ITextModel> | null;
    private readonly modelDisposable;
    protected model: ITextModel | null;
    private modelUpdateInProgress;
    private readonly modelUpdateCancellationSource;
    private readonly appendThrottler;
    private replacePromise;
    abstract readonly source: IOutputContentSource | ReadonlyArray<IOutputContentSource>;
    constructor(modelUri: URI, language: ILanguageSelection, outputContentProvider: IContentProvider, modelService: IModelService, editorWorkerService: IEditorWorkerService);
    loadModel(): Promise<ITextModel>;
    getLogEntries(): readonly ILogEntry[];
    private onDidContentChange;
    protected doUpdate(mode: OutputChannelUpdateMode, immediate: boolean): void;
    private clearContent;
    private appendContent;
    private doAppendContent;
    private replaceContent;
    private getReplaceEdits;
    protected cancelModelUpdate(): void;
    protected isVisible(): boolean;
    dispose(): void;
    append(message: string): void;
    replace(message: string): void;
    abstract clear(): void;
    abstract update(mode: OutputChannelUpdateMode, till: number | undefined, immediate: boolean): void;
    abstract updateChannelSources(files: IOutputContentSource[]): void;
}
export declare class FileOutputChannelModel extends AbstractFileOutputChannelModel implements IOutputChannelModel {
    readonly source: IOutputContentSource;
    private readonly fileOutput;
    constructor(modelUri: URI, language: ILanguageSelection, source: IOutputContentSource, fileService: IFileService, modelService: IModelService, instantiationService: IInstantiationService, logService: ILogService, editorWorkerService: IEditorWorkerService);
    clear(): void;
    update(mode: OutputChannelUpdateMode, till: number | undefined, immediate: boolean): void;
    updateChannelSources(files: IOutputContentSource[]): void;
}
export declare class MultiFileOutputChannelModel extends AbstractFileOutputChannelModel implements IOutputChannelModel {
    readonly source: IOutputContentSource[];
    private readonly multifileOutput;
    constructor(modelUri: URI, language: ILanguageSelection, source: IOutputContentSource[], fileService: IFileService, modelService: IModelService, logService: ILogService, editorWorkerService: IEditorWorkerService, instantiationService: IInstantiationService);
    updateChannelSources(files: IOutputContentSource[]): void;
    clear(): void;
    update(mode: OutputChannelUpdateMode, till: number | undefined, immediate: boolean): void;
}
export declare class DelegatedOutputChannelModel extends Disposable implements IOutputChannelModel {
    private readonly instantiationService;
    private readonly fileService;
    private readonly _onDispose;
    readonly onDispose: Event<void>;
    private readonly outputChannelModel;
    readonly source: IOutputContentSource;
    constructor(id: string, modelUri: URI, language: ILanguageSelection, outputDir: URI, outputDirCreationPromise: Promise<void>, instantiationService: IInstantiationService, fileService: IFileService);
    private createOutputChannelModel;
    getLogEntries(): readonly ILogEntry[];
    append(output: string): void;
    update(mode: OutputChannelUpdateMode, till: number | undefined, immediate: boolean): void;
    loadModel(): Promise<ITextModel>;
    clear(): void;
    replace(value: string): void;
    updateChannelSources(files: IOutputContentSource[]): void;
}
export {};
